@@ -40,6 +40,7 @@
 #include "katepartdebug.h"
 #include "katecommandrangeexpressionparser.h"
 #include "kateabstractinputmode.h"
+#include "katetextpreview.h"
 
 #include <KCharsets>
 #include <KColorUtils>
@@ -101,6 +102,7 @@ KateScrollBar::KateScrollBar(Qt::Orientation orientation, KateViewInternal *pare
     , m_view(parent->m_view)
     , m_doc(parent->doc())
     , m_viewInternal(parent)
+    , m_textPreview(nullptr)
     , m_showMarks(false)
     , m_showMiniMap(false)
     , m_miniMapAll(true)
@@ -114,6 +116,19 @@ KateScrollBar::KateScrollBar(Qt::Orientation orientation, KateViewInternal *pare
     m_updateTimer.setInterval(300);
     m_updateTimer.setSingleShot(true);
     QTimer::singleShot(10, this, SLOT(updatePixmap()));
+
+    // track mouse for text preview widget
+    setMouseTracking(orientation == Qt::Vertical);
+
+    // setup text preview timer
+    m_delayTextPreviewTimer.setSingleShot(true);
+    m_delayTextPreviewTimer.setInterval(250);
+    connect(&m_delayTextPreviewTimer, SIGNAL(timeout()), this, SLOT(showTextPreview()));
+}
+
+KateScrollBar::~KateScrollBar()
+{
+    delete m_textPreview;
 }
 
 void KateScrollBar::setShowMiniMap(bool b)
@@ -174,6 +189,9 @@ int KateScrollBar::minimapYToStdY(int y)
 
 void KateScrollBar::mousePressEvent(QMouseEvent *e)
 {
+    // delete text preview
+    hideTextPreview();
+
     if (m_showMiniMap) {
         QMouseEvent eMod(QEvent::MouseButtonPress,
                          QPoint(6, minimapYToStdY(e->pos().y())),
@@ -241,6 +259,27 @@ void KateScrollBar::mouseMoveEvent(QMouseEvent *e)
         const int lastLine = m_viewInternal->toRealCursor(m_viewInternal->endPos()).line() + 1;
         QToolTip::showText(m_toolTipPos, i18nc("from line - to line", "<center>%1<br/>&#x2014;<br/>%2</center>", fromLine, lastLine), this);
     }
+
+    showTextPreviewDelayed();
+}
+
+void KateScrollBar::leaveEvent(QEvent *event)
+{
+    hideTextPreview();
+
+    QAbstractSlider::leaveEvent(event);
+}
+
+bool KateScrollBar::eventFilter(QObject *object, QEvent *event)
+{
+    Q_UNUSED(object)
+
+    if (m_textPreview && event->type() == QEvent::WindowDeactivate) {
+        // We need hide the scrollbar TextPreview widget
+        hideTextPreview();
+    }
+
+    return false;
 }
 
 void KateScrollBar::paintEvent(QPaintEvent *e)
@@ -255,6 +294,89 @@ void KateScrollBar::paintEvent(QPaintEvent *e)
     }
 }
 
+void KateScrollBar::showTextPreviewDelayed()
+{
+    if (!m_textPreview) {
+        if (!m_delayTextPreviewTimer.isActive()) {
+            m_delayTextPreviewTimer.start();
+        }
+    } else {
+        showTextPreview();
+    }
+}
+
+void KateScrollBar::showTextPreview()
+{
+    if (orientation() != Qt::Vertical || isSliderDown() || (minimum() == maximum()) || !m_view->config()->scrollBarPreview()) {
+        return;
+    }
+
+    QRect grooveRect;
+    if (m_showMiniMap) {
+        // If mini-map is shown, the height of the map might not be the whole height
+        grooveRect = m_mapGroveRect;
+    } else {
+        QStyleOptionSlider opt;
+        opt.init(this);
+        opt.subControls = QStyle::SC_None;
+        opt.activeSubControls = QStyle::SC_None;
+        opt.orientation = orientation();
+        opt.minimum = minimum();
+        opt.maximum = maximum();
+        opt.sliderPosition = sliderPosition();
+        opt.sliderValue = value();
+        opt.singleStep = singleStep();
+        opt.pageStep = pageStep();
+
+        grooveRect = style()->subControlRect(QStyle::CC_ScrollBar, &opt, QStyle::SC_ScrollBarGroove, this);
+    }
+
+    if (m_view->config()->scrollPastEnd()) {
+        // Adjust the grove size to accommodate the added pageStep at the bottom
+        int adjust = pageStep()*grooveRect.height() / (maximum() + pageStep() - minimum());
+        grooveRect.adjust(0,0,0, -adjust);
+    }
+
+    const QPoint cursorPos = mapFromGlobal(QCursor::pos());
+    if (grooveRect.contains(cursorPos)) {
+
+        if (!m_textPreview) {
+            m_textPreview = new KateTextPreview(m_view);
+            m_textPreview->setAttribute(Qt::WA_ShowWithoutActivating);
+            m_textPreview->setFrameStyle(QFrame::StyledPanel);
+
+            // event filter to catch application WindowDeactivate event, to hide the preview window
+            qApp->installEventFilter(this);
+        }
+
+        const qreal posInPercent = static_cast<double>(cursorPos.y() - grooveRect.top()) / grooveRect.height();
+        const qreal startLine = posInPercent * m_view->textFolding().visibleLines();
+
+        m_textPreview->resize(m_view->width() / 2, m_view->height() / 5);
+        const int xGlobal = mapToGlobal(QPoint(0, 0)).x();
+        const int yGlobal = qMin(mapToGlobal(QPoint(0, height())).y() - m_textPreview->height(),
+                                 qMax(mapToGlobal(QPoint(0, 0)).y(), mapToGlobal(cursorPos).y() - m_textPreview->height() / 2));
+        m_textPreview->move(xGlobal - m_textPreview->width(), yGlobal);
+        m_textPreview->setLine(startLine);
+        m_textPreview->setCenterView(true);
+        m_textPreview->setScaleFactor(0.8);
+        m_textPreview->raise();
+        m_textPreview->show();
+    } else {
+        hideTextPreview();
+    }
+}
+
+void KateScrollBar::hideTextPreview()
+{
+    if (m_delayTextPreviewTimer.isActive()) {
+        m_delayTextPreviewTimer.stop();
+    }
+
+    qApp->removeEventFilter(this);
+    delete m_textPreview;
+}
+
 // This function is optimized for bing called in sequence.
 const QColor KateScrollBar::charColor(const QVector<Kate::TextLineData::Attribute> &attributes, int &attributeIndex,
                                       const QList<QTextLayout::FormatRange> &decorations,
@@ -347,13 +469,15 @@ void KateScrollBar::updatePixmap()
 
     const QColor backgroundColor = m_view->defaultStyleAttribute(KTextEditor::dsNormal)->background().color();
     const QColor defaultTextColor = m_view->defaultStyleAttribute(KTextEditor::dsNormal)->foreground().color();
+    const QColor selectionBgColor = m_view->renderer()->config()->selectionColor();
     QColor modifiedLineColor = m_view->renderer()->config()->modifiedLineColor();
     QColor savedLineColor = m_view->renderer()->config()->savedLineColor();
     // move the modified line color away from the background color
     modifiedLineColor.setHsv(modifiedLineColor.hue(), 255, 255 - backgroundColor.value() / 3);
     savedLineColor.setHsv(savedLineColor.hue(), 100, 255 - backgroundColor.value() / 3);
 
-    m_pixmap = QPixmap(pixmapLineWidth, pixmapLineCount);
+    // increase dimensions by ratio
+    m_pixmap = QPixmap(pixmapLineWidth * m_view->devicePixelRatio(), pixmapLineCount * m_view->devicePixelRatio());
     m_pixmap.fill(QColor("transparent"));
 
     // The text currently selected in the document, to be drawn later.
@@ -361,6 +485,9 @@ void KateScrollBar::updatePixmap()
 
     QPainter painter;
     if (painter.begin(&m_pixmap)) {
+        // init pen once, afterwards, only change it if color changes to avoid a lot of allocation for setPen
+        painter.setPen(selectionBgColor);
+
         // Do not force updates of the highlighting if the document is very large
         bool simpleMode = m_doc->lines() > 7500;
 
@@ -373,9 +500,6 @@ void KateScrollBar::updatePixmap()
             int realLineNumber = m_view->textFolding().visibleLineToLine(virtualLine);
             QString lineText = m_doc->line(realLineNumber);
 
-            // use this to control the offset of the text from the left
-            int pixelX = s_pixelMargin;
-
             if (!simpleMode) {
                 m_doc->buffer().ensureHighlighted(realLineNumber);
             }
@@ -385,13 +509,47 @@ void KateScrollBar::updatePixmap()
             QList< QTextLayout::FormatRange > decorations = m_view->renderer()->decorationsForLine(kateline, realLineNumber);
             int attributeIndex = 0;
 
-            // The color to draw the currently selected text in; change the alpha value to make it
-            // more or less intense
-            QColor selectionColor = palette().color(QPalette::HighlightedText);
-            selectionColor.setAlpha(180);
+            // Draw selection if it is on an empty line
+            if (selection.contains(KTextEditor::Cursor(realLineNumber, 0)) && lineText.size() == 0) {
+                if (selectionBgColor != painter.pen().color()) {
+                    painter.setPen(selectionBgColor);
+                }
+                painter.drawLine(s_pixelMargin, pixelY, s_pixelMargin + s_lineWidth - 1, pixelY);
+            }
+
+            // Iterate over the line to draw the background
+            int selStartX = -1;
+            int selEndX = -1;
+            int pixelX = s_pixelMargin; // use this to control the offset of the text from the left
+            for (int x = 0; (x < lineText.size() && x < s_lineWidth); x += charIncrement) {
+                if (pixelX >= s_lineWidth + s_pixelMargin) {
+                    break;
+                }
+                // Query the selection and draw it behind the character
+                if (selection.contains(KTextEditor::Cursor(realLineNumber, x))) {
+                    if (selStartX == -1) selStartX = pixelX;
+                    selEndX = pixelX;
+                    if (lineText.size() - 1 == x) {
+                        selEndX = s_lineWidth + s_pixelMargin-1;
+                    }
+                }
+
+                if (lineText[x] == QLatin1Char('\t')) {
+                    pixelX += qMax(4 / charIncrement, 1); // FIXME: tab width...
+                } else {
+                    pixelX++;
+                }
+            }
+
+            if (selStartX != -1) {
+                if (selectionBgColor != painter.pen().color()) {
+                    painter.setPen(selectionBgColor);
+                }
+                painter.drawLine(selStartX, pixelY, selEndX, pixelY);
+            }
 
-            painter.setPen(defaultTextColor);
             // Iterate over all the characters in the current line
+            pixelX = s_pixelMargin;
             for (int x = 0; (x < lineText.size() && x < s_lineWidth); x += charIncrement) {
                 if (pixelX >= s_lineWidth + s_pixelMargin) {
                     break;
@@ -403,7 +561,10 @@ void KateScrollBar::updatePixmap()
                 } else if (lineText[x] == QLatin1Char('\t')) {
                     pixelX += qMax(4 / charIncrement, 1); // FIXME: tab width...
                 } else {
-                    painter.setPen(charColor(attributes, attributeIndex, decorations, defaultTextColor, x, lineText[x]));
+                    const QColor newPenColor(charColor(attributes, attributeIndex, decorations, defaultTextColor, x, lineText[x]));
+                    if (newPenColor != painter.pen().color()) {
+                        painter.setPen(newPenColor);
+                    }
 
                     // Actually draw the pixel with the color queried from the renderer.
                     painter.drawPoint(pixelX, pixelY);
@@ -411,17 +572,6 @@ void KateScrollBar::updatePixmap()
                     pixelX++;
                 }
 
-                // Query the selection and draw it above the character with an alpha channel
-                if (selection.contains(KTextEditor::Cursor(realLineNumber, x))) {
-                    painter.setPen(selectionColor);
-                    painter.drawPoint(s_pixelMargin, pixelY);
-                    // fill the line up in case the selection extends beyond it
-                    if (lineText.size() - 1 == x) {
-                        for (int xFill = s_pixelMargin; xFill < s_lineWidth; xFill++) {
-                            painter.drawPoint(xFill, pixelY);
-                        }
-                    }
-                }
             }
             drawnLines++;
             if (((drawnLines) % charIncrement) == 0) {
@@ -437,17 +587,20 @@ void KateScrollBar::updatePixmap()
             for (int lineno = 0; lineno < docLineCount; lineno++) {
                 int realLineNo = m_view->textFolding().visibleLineToLine(lineno);
                 const Kate::TextLine &line = m_doc->plainKateTextLine(realLineNo);
-                if (line->markedAsModified()) {
-                    painter.setPen(modifiedLineColor);
-                } else if (line->markedAsSavedOnDisk()) {
-                    painter.setPen(savedLineColor);
-                } else {
-                    continue;
+                const QColor & col = line->markedAsModified() ? modifiedLineColor : savedLineColor;
+                if (line->markedAsModified() || line->markedAsSavedOnDisk()) {
+                    painter.fillRect(2, lineno / lineDivisor, 3, 1, col);
                 }
-                painter.drawRect(2, lineno / lineDivisor, 3, 1);
             }
         }
+
+        // end painting
+        painter.end();
     }
+
+    // set right ratio
+    m_pixmap.setDevicePixelRatio(m_view->devicePixelRatio());
+
     //qCDebug(LOG_KTE) << time.elapsed();
     // Redraw the scrollbar widget with the updated pixmap.
     update();
@@ -471,7 +624,6 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
     opt.singleStep = singleStep();
     opt.pageStep = pageStep();
 
-    int docXMargin = 1;
     QRect grooveRect = style()->subControlRect(QStyle::CC_ScrollBar, &opt, QStyle::SC_ScrollBarGroove, this);
     m_stdGroveRect = grooveRect;
     if (style()->subControlRect(QStyle::CC_ScrollBar, &opt, QStyle::SC_ScrollBarSubLine, this).height() == 0) {
@@ -485,17 +637,17 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
     }
     m_grooveHeight = grooveRect.height();
 
+    const int docXMargin = 1;
     QRect sliderRect = style()->subControlRect(QStyle::CC_ScrollBar, &opt, QStyle::SC_ScrollBarSlider, this);
-    m_stdSliderRect = sliderRect;
-    sliderRect.adjust(docXMargin + 1, 1, -(docXMargin + 1), -1);
+    sliderRect.adjust(docXMargin, 0, 0, 0);
 
     //style()->drawControl(QStyle::CE_ScrollBarAddLine, &opt, &painter, this);
     //style()->drawControl(QStyle::CE_ScrollBarSubLine, &opt, &painter, this);
 
     // calculate the document size and position
-    int docHeight = qMin(grooveRect.height(), m_pixmap.height() * 2) - 2 * docXMargin;
-    int yoffset = 1; // top-aligned in stead of center-aligned (grooveRect.height() - docHeight) / 2;
-    QRect docRect(QPoint(grooveRect.left() + docXMargin, yoffset + grooveRect.top()), QSize(grooveRect.width() - 2 * docXMargin, docHeight));
+    const int docHeight = qMin(grooveRect.height(), int(m_pixmap.height() / m_pixmap.devicePixelRatio() * 2)) - 2 * docXMargin;
+    const int yoffset = 1; // top-aligned in stead of center-aligned (grooveRect.height() - docHeight) / 2;
+    const QRect docRect(QPoint(grooveRect.left() + docXMargin, yoffset + grooveRect.top()), QSize(grooveRect.width() - docXMargin, docHeight));
     m_mapGroveRect = docRect;
 
     // calculate the visible area
@@ -512,9 +664,9 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
     const QColor foregroundColor = m_view->defaultStyleAttribute(KTextEditor::dsNormal)->foreground().color();
     const QColor highlightColor  = palette().link().color();
 
-    int backgroundLightness = backgroundColor.lightness();
-    int foregroundLightness = foregroundColor.lightness();
-    int lighnessDiff = (foregroundLightness - backgroundLightness);
+    const int backgroundLightness = backgroundColor.lightness();
+    const int foregroundLightness = foregroundColor.lightness();
+    const int lighnessDiff = (foregroundLightness - backgroundLightness);
 
     // get a color suited for the color theme
     QColor darkShieldColor = palette().color(QPalette::Mid);
@@ -544,21 +696,22 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
         visibleRect.adjust(2, 0, -3, 0);
     } else {
         visibleRect.adjust(1, 0, -1, 2);
-        sliderRect = visibleRect;
+        sliderRect.setTop(visibleRect.top() - 1);
+        sliderRect.setBottom(visibleRect.bottom() + 1);
     }
 
     // Smooth transform only when squeezing
-    if (grooveRect.height() < m_pixmap.height()) {
+    if (grooveRect.height() < m_pixmap.height() / m_pixmap.devicePixelRatio()) {
         painter.setRenderHint(QPainter::SmoothPixmapTransform);
     }
 
     // draw the modified lines margin
-    QRect pixmapMarginRect(QPoint(0, 0), QSize(s_pixelMargin, m_pixmap.height()));
+    QRect pixmapMarginRect(QPoint(0, 0), QSize(s_pixelMargin, m_pixmap.height() / m_pixmap.devicePixelRatio()));
     QRect docPixmapMarginRect(QPoint(0, docRect.top()), QSize(s_pixelMargin, docRect.height()));
     painter.drawPixmap(docPixmapMarginRect, m_pixmap, pixmapMarginRect);
 
     // calculate the stretch and draw the stretched lines (scrollbar marks)
-    QRect pixmapRect(QPoint(s_pixelMargin, 0), QSize(m_pixmap.width() - s_pixelMargin, m_pixmap.height()));
+    QRect pixmapRect(QPoint(s_pixelMargin, 0), QSize(m_pixmap.width() / m_pixmap.devicePixelRatio() - s_pixelMargin, m_pixmap.height() / m_pixmap.devicePixelRatio()));
     QRect docPixmapRect(QPoint(s_pixelMargin, docRect.top()), QSize(docRect.width() - s_pixelMargin, docRect.height()));
     painter.drawPixmap(docPixmapRect, m_pixmap, pixmapRect);
 
@@ -569,7 +722,7 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
         fg.setAlpha(30);
         painter.setBrush(Qt::NoBrush);
         painter.setPen(QPen(fg, 1));
-        painter.drawLine(grooveRect.x()+1,y+4,width()-1,y+4);
+        painter.drawLine(grooveRect.x()+1,y+2,width()-1,y+2);
     }
 
     // fade the invisible sections
@@ -588,12 +741,10 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
 
     QColor faded(backgroundColor);
     faded.setAlpha(110);
-    painter.setPen(Qt::NoPen);
-    painter.setBrush(faded);
-    painter.drawRect(top);
-    painter.drawRect(bottom);
+    painter.fillRect(top, faded);
+    painter.fillRect(bottom, faded);
 
-    // add a thin line to delimitate the scrollbar
+    // add a thin line to limit the scrollbar
     QColor c(foregroundColor);
     c.setAlpha(10);
     painter.setPen(QPen(c,1));
@@ -627,11 +778,13 @@ void KateScrollBar::miniMapPaintEvent(QPaintEvent *e)
     // slider outline
     QColor sliderColor(highlightColor);
     sliderColor.setAlpha(50);
-    painter.setRenderHint(QPainter::Antialiasing);
-    painter.setPen(QPen(highlightColor,1));
-    painter.setBrush(sliderColor);
-    painter.drawRoundedRect(sliderRect, 3, 3);
-    painter.setRenderHint(QPainter::Antialiasing, false);
+    painter.fillRect(sliderRect, sliderColor);
+    painter.setPen(QPen(highlightColor, 0));
+    // rounded rect looks ugly for some reason, so we draw 4 lines.
+    painter.drawLine(sliderRect.left(), sliderRect.top() + 1, sliderRect.left(), sliderRect.bottom() - 1);
+    painter.drawLine(sliderRect.right(), sliderRect.top() + 1, sliderRect.right(), sliderRect.bottom() - 1);
+    painter.drawLine(sliderRect.left() + 1, sliderRect.top(), sliderRect.right() - 1, sliderRect.top());
+    painter.drawLine(sliderRect.left() + 1, sliderRect.bottom(), sliderRect.right() - 1, sliderRect.bottom());
 }
 
 void KateScrollBar::normalPaintEvent(QPaintEvent *e)
@@ -797,7 +950,7 @@ KateCommandLineBar::KateCommandLineBar(KTextEditor::ViewPrivate *view, QWidget *
 
     QToolButton *helpButton = new QToolButton(this);
     helpButton->setAutoRaise(true);
-    helpButton->setIcon(QIcon::fromTheme(QLatin1String("help-contextual")));
+    helpButton->setIcon(QIcon::fromTheme(QStringLiteral("help-contextual")));
     topLayout->addWidget(helpButton);
     connect(helpButton, SIGNAL(clicked()), this, SLOT(showHelpPage()));
 
@@ -806,7 +959,7 @@ KateCommandLineBar::KateCommandLineBar(KTextEditor::ViewPrivate *view, QWidget *
 
 void KateCommandLineBar::showHelpPage()
 {
-    KHelpClient::invokeHelp(QLatin1String("advanced-editing-tools-commandline"), QLatin1String("kate"));
+    KHelpClient::invokeHelp(QStringLiteral("advanced-editing-tools-commandline"), QStringLiteral("kate"));
 }
 
 KateCommandLineBar::~KateCommandLineBar()
@@ -835,7 +988,7 @@ KateCmdLineEdit::KateCmdLineEdit(KateCommandLineBar *bar, KTextEditor::ViewPriva
     , m_msgMode(false)
     , m_histpos(0)
     , m_cmdend(0)
-    , m_command(0L)
+    , m_command(nullptr)
 {
     connect(this, SIGNAL(returnPressed(QString)),
             this, SLOT(slotReturnPressed(QString)));
@@ -861,9 +1014,9 @@ void KateCmdLineEdit::hideEvent(QHideEvent *e)
 
 QString KateCmdLineEdit::helptext(const QPoint &) const
 {
-    QString beg = QString::fromLatin1("<qt background=\"white\"><div><table width=\"100%\"><tr><td bgcolor=\"brown\"><font color=\"white\"><b>Help: <big>");
-    QString mid = QString::fromLatin1("</big></b></font></td></tr><tr><td>");
-    QString end = QString::fromLatin1("</td></tr></table></div><qt>");
+    QString beg = QStringLiteral("<qt background=\"white\"><div><table width=\"100%\"><tr><td bgcolor=\"brown\"><font color=\"white\"><b>Help: <big>");
+    QString mid = QStringLiteral("</big></b></font></td></tr><tr><td>");
+    QString end = QStringLiteral("</td></tr></table></div><qt>");
 
     QString t = text();
     QRegExp re(QLatin1String("\\s*help\\s+(.*)"));
@@ -873,7 +1026,7 @@ QString KateCmdLineEdit::helptext(const QPoint &) const
         QString name = re.cap(1);
         if (name == QLatin1String("list")) {
             return beg + i18n("Available Commands") + mid
-                   + KateCmd::self()->commandList().join(QLatin1String(" "))
+                   + KateCmd::self()->commandList().join(QLatin1Char(' '))
                    + i18n("<p>For help on individual commands, do <code>'help &lt;command&gt;'</code></p>")
                    + end;
         } else if (! name.isEmpty()) {
@@ -972,7 +1125,7 @@ void KateCmdLineEdit::slotReturnPressed(const QString &text)
         m_msgMode = true;
 
         // the following commands changes the focus themselves, so bar should be hidden before execution.
-        if (QRegExp(QLatin1String("buffer|b|new|vnew|bp|bprev|bn|bnext|bf|bfirst|bl|blast|edit|e")).exactMatch(cmd.split(QLatin1String(" ")).at(0))) {
+        if (QRegExp(QLatin1String("buffer|b|new|vnew|bp|bprev|bn|bnext|bf|bfirst|bl|blast|edit|e")).exactMatch(cmd.split(QLatin1Char(' ')).at(0))) {
             emit hideRequested();
         }
 
@@ -1018,11 +1171,11 @@ void KateCmdLineEdit::slotReturnPressed(const QString &text)
         setCompletionObject(KateCmd::self()->commandCompletionObject());
         delete c;
     }
-    m_command = 0;
+    m_command = nullptr;
     m_cmdend = 0;
 
     // the following commands change the focus themselves
-    if (!QRegExp(QLatin1String("buffer|b|new|vnew|bp|bprev|bn|bnext|bf|bfirst|bl|blast|edit|e")).exactMatch(cmd.split(QLatin1String(" ")).at(0))) {
+    if (!QRegExp(QLatin1String("buffer|b|new|vnew|bp|bprev|bn|bnext|bf|bfirst|bl|blast|edit|e")).exactMatch(cmd.split(QLatin1Char(' ')).at(0))) {
         m_view->setFocus();
     }
 
@@ -1069,7 +1222,7 @@ void KateCmdLineEdit::keyPressEvent(QKeyEvent *ev)
     if (! m_cmdend || cursorpos <= m_cmdend) {
         QChar c;
         if (! ev->text().isEmpty()) {
-            c = ev->text()[0];
+            c = ev->text().at(0);
         }
 
         if (! m_cmdend && ! c.isNull()) { // we have no command, so lets see if we got one
@@ -1173,8 +1326,6 @@ void KateCmdLineEdit::fromHistory(bool up)
 //BEGIN KateIconBorder
 using namespace KTextEditor;
 
-const int halfIPW = 8;
-
 KateIconBorder::KateIconBorder(KateViewInternal *internalView, QWidget *parent)
     : QWidget(parent)
     , m_view(internalView->m_view)
@@ -1193,7 +1344,8 @@ KateIconBorder::KateIconBorder(KateViewInternal *internalView, QWidget *parent)
     , m_maxCharWidth(0.0)
     , iconPaneWidth(16)
     , m_annotationBorderWidth(6)
-    , m_foldingRange(0)
+    , m_foldingPreview(nullptr)
+    , m_foldingRange(nullptr)
     , m_nextHighlightBlock(-2)
     , m_currentBlockLine(-1)
 {
@@ -1201,19 +1353,22 @@ KateIconBorder::KateIconBorder(KateViewInternal *internalView, QWidget *parent)
     setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Minimum);
     setMouseTracking(true);
     m_doc->setMarkDescription(MarkInterface::markType01, i18n("Bookmark"));
-    m_doc->setMarkPixmap(MarkInterface::markType01, QIcon::fromTheme(QLatin1String("bookmarks")).pixmap(16, 16));
+    m_doc->setMarkPixmap(MarkInterface::markType01, QIcon::fromTheme(QStringLiteral("bookmarks")).pixmap(16, 16));
 
     updateFont();
 
     m_delayFoldingHlTimer.setSingleShot(true);
     m_delayFoldingHlTimer.setInterval(150);
     connect(&m_delayFoldingHlTimer, SIGNAL(timeout()), this, SLOT(showBlock()));
+
+    // user interaction (scrolling) hides e.g. preview
+    connect(m_view, SIGNAL(displayRangeChanged(KTextEditor::ViewPrivate*)), this, SLOT(displayRangeChanged()));
 }
 
 KateIconBorder::~KateIconBorder()
 {
+    delete m_foldingPreview;
     delete m_foldingRange;
-    m_foldingRange = 0;
 }
 
 void KateIconBorder::setIconBorderOn(bool enable)
@@ -1634,10 +1789,10 @@ void KateIconBorder::paintBorder(int /*x*/, int y, int /*width*/, int height)
                     if (m_relLineNumbersOn) {
                         if (distanceToCurrent == 0) {
                             p.drawText(lnX + m_maxCharWidth / 2, y, lnWidth - m_maxCharWidth, h,
-                                       Qt::TextDontClip|Qt::AlignLeft|Qt::AlignVCenter, QString::fromLatin1("%1").arg(realLine + 1));
+                                       Qt::TextDontClip|Qt::AlignLeft|Qt::AlignVCenter, QString::number(realLine + 1));
                         } else {
                             p.drawText(lnX + m_maxCharWidth / 2, y, lnWidth - m_maxCharWidth, h,
-                                       Qt::TextDontClip|Qt::AlignRight|Qt::AlignVCenter, QString::fromLatin1("%1").arg(distanceToCurrent));
+                                       Qt::TextDontClip|Qt::AlignRight|Qt::AlignVCenter, QString::number(distanceToCurrent));
                         }
                         if (m_updateRelLineNumbers) {
                             m_updateRelLineNumbers = false;
@@ -1645,7 +1800,7 @@ void KateIconBorder::paintBorder(int /*x*/, int y, int /*width*/, int height)
                         }
                     } else if (m_lineNumbersOn) {
                         p.drawText(lnX + m_maxCharWidth / 2, y, lnWidth - m_maxCharWidth, h,
-                                   Qt::TextDontClip | Qt::AlignRight | Qt::AlignVCenter, QString::fromLatin1("%1").arg(realLine + 1));
+                                   Qt::TextDontClip | Qt::AlignRight | Qt::AlignVCenter, QString::number(realLine + 1));
                     }
                 } else if (m_view->dynWordWrap() && m_dynWrapIndicatorsOn) {
                     p.drawPixmap(lnX + lnWidth - (m_arrow.width() / m_arrow.devicePixelRatio()) - 2, y, m_arrow);
@@ -1837,7 +1992,7 @@ void KateIconBorder::showBlock()
         return;
     } else { // the ranges differ, delete the old, if it exists
         delete m_foldingRange;
-        m_foldingRange = 0;
+        m_foldingRange = nullptr;
     }
 
     if (newRange.isValid()) {
@@ -1858,6 +2013,50 @@ void KateIconBorder::showBlock()
         m_foldingRange->setAttribute(attr);
     }
 
+    // show text preview, if a folded region starts here
+    bool foldUnderMouse = false;
+    if (m_foldingRange && m_view->config()->foldingPreview()) {
+        const QPoint globalPos = QCursor::pos();
+        const QPoint pos = mapFromGlobal(globalPos);
+        const KateTextLayout &t = m_view->m_viewInternal->yToKateTextLayout(pos.y());
+        if (t.isValid() && positionToArea(pos) == FoldingMarkers) {
+
+            const int realLine = t.line();
+            foldUnderMouse = !m_view->textFolding().isLineVisible(realLine + 1);
+
+            if (foldUnderMouse) {
+                if (!m_foldingPreview) {
+                    m_foldingPreview = new KateTextPreview(m_view);
+                    m_foldingPreview->setAttribute(Qt::WA_ShowWithoutActivating);
+                    m_foldingPreview->setFrameStyle(QFrame::StyledPanel);
+
+                    // event filter to catch application WindowDeactivate event, to hide the preview window
+//                     qApp->installEventFilter(this);
+                }
+
+                // TODO: use KateViewInternal::maxLen() somehow to compute proper width for amount of lines to display
+
+                // try using the end line of the range for proper popup height
+                const int lineCount = qMin(m_foldingRange->numberOfLines() + 1,
+                                           (height() - pos.y()) / m_view->renderer()->lineHeight());
+
+                m_foldingPreview->resize(m_view->width() / 2, lineCount * m_view->renderer()->lineHeight() + 2 * m_foldingPreview->frameWidth());
+                const int xGlobal = mapToGlobal(QPoint(width(), 0)).x();
+                const int yGlobal = m_view->mapToGlobal(m_view->cursorToCoordinate(KTextEditor::Cursor(realLine, 0))).y();
+                m_foldingPreview->move(QPoint(xGlobal, yGlobal) - m_foldingPreview->contentsRect().topLeft());
+                m_foldingPreview->setLine(realLine);
+                m_foldingPreview->setCenterView(false);
+                m_foldingPreview->setShowFoldedLines(true);
+                m_foldingPreview->raise();
+                m_foldingPreview->show();
+            }
+        }
+    }
+
+    if (!foldUnderMouse) {
+        delete m_foldingPreview;
+    }
+
     /**
      * repaint
      */
@@ -1873,7 +2072,9 @@ void KateIconBorder::hideBlock()
     m_nextHighlightBlock = -2;
     m_currentBlockLine = -1;
     delete m_foldingRange;
-    m_foldingRange = 0;
+    m_foldingRange = nullptr;
+
+    delete m_foldingPreview;
 }
 
 void KateIconBorder::leaveEvent(QEvent *event)
@@ -1972,10 +2173,12 @@ void KateIconBorder::mouseReleaseEvent(QMouseEvent *e)
                 }
                 m_view->textFolding().newFoldingRange(foldingRange, Kate::TextFolding::Folded);
             }
+
+            delete m_foldingPreview;
         }
 
         if (area == AnnotationBorder) {
-            const bool singleClick = style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, this);
+            const bool singleClick = style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, this);
             if (e->button() == Qt::LeftButton && singleClick) {
                 emit m_view->annotationActivated(m_view, cursorOnLine);
             } else if (e->button() == Qt::RightButton) {
@@ -1996,7 +2199,7 @@ void KateIconBorder::mouseDoubleClickEvent(QMouseEvent *e)
     if (cursorOnLine == m_lastClickedLine &&
             cursorOnLine <= m_doc->lastLine()) {
         BorderArea area = positionToArea(e->pos());
-        const bool singleClick = style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, this);
+        const bool singleClick = style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, this);
         if (area == AnnotationBorder && !singleClick) {
             emit m_view->annotationActivated(m_view, cursorOnLine);
         }
@@ -2122,7 +2325,7 @@ void KateIconBorder::showAnnotationMenu(int line, const QPoint &pos)
 {
     QMenu menu;
     QAction a(i18n("Disable Annotation Bar"), &menu);
-    a.setIcon(QIcon::fromTheme(QLatin1String("dialog-close")));
+    a.setIcon(QIcon::fromTheme(QStringLiteral("dialog-close")));
     menu.addAction(&a);
     emit m_view->annotationContextMenuAboutToShow(m_view, &menu, line);
     if (menu.exec(pos) == &a) {
@@ -2167,6 +2370,12 @@ void KateIconBorder::annotationModelChanged(KTextEditor::AnnotationModel *oldmod
     updateAnnotationBorderWidth();
 }
 
+void KateIconBorder::displayRangeChanged()
+{
+    hideBlock();
+    removeAnnotationHovering();
+}
+
 //END KateIconBorder
 
 //BEGIN KateViewEncodingAction
@@ -2353,7 +2562,7 @@ bool KateViewEncodingAction::setCurrentCodec(int mib)
 
 KateViewBarWidget::KateViewBarWidget(bool addCloseButton, QWidget *parent)
     : QWidget(parent)
-    , m_viewBar(0)
+    , m_viewBar(nullptr)
 {
     QHBoxLayout *layout = new QHBoxLayout(this);
 
@@ -2364,7 +2573,7 @@ KateViewBarWidget::KateViewBarWidget(bool addCloseButton, QWidget *parent)
     if (addCloseButton) {
         QToolButton *hideButton = new QToolButton(this);
         hideButton->setAutoRaise(true);
-        hideButton->setIcon(QIcon::fromTheme(QLatin1String("dialog-close")));
+        hideButton->setIcon(QIcon::fromTheme(QStringLiteral("dialog-close")));
         connect(hideButton, SIGNAL(clicked()), SIGNAL(hideMe()));
         layout->addWidget(hideButton);
         layout->setAlignment(hideButton, Qt::AlignLeft | Qt::AlignTop);
@@ -2379,7 +2588,7 @@ KateViewBarWidget::KateViewBarWidget(bool addCloseButton, QWidget *parent)
 }
 
 KateViewBar::KateViewBar(bool external, QWidget *parent, KTextEditor::ViewPrivate *view)
-    : QWidget(parent), m_external(external), m_view(view), m_permanentBarWidget(0)
+    : QWidget(parent), m_external(external), m_view(view), m_permanentBarWidget(nullptr)
 
 {
     m_layout = new QVBoxLayout(this);
@@ -2413,9 +2622,9 @@ void KateViewBar::removeBarWidget(KateViewBarWidget *barWidget)
     }
 
     m_stack->removeWidget(barWidget);
-    barWidget->setAssociatedViewBar(0);
+    barWidget->setAssociatedViewBar(nullptr);
     barWidget->hide();
-    disconnect(barWidget, 0, this, 0);
+    disconnect(barWidget, nullptr, this, nullptr);
 }
 
 void KateViewBar::addPermanentBarWidget(KateViewBarWidget *barWidget)
@@ -2441,7 +2650,7 @@ void KateViewBar::removePermanentBarWidget(KateViewBarWidget *barWidget)
 
     m_permanentBarWidget->hide();
     m_stack->removeWidget(m_permanentBarWidget);
-    m_permanentBarWidget = 0;
+    m_permanentBarWidget = nullptr;
 
     if (hideBar) {
         m_stack->hide();
@@ -2456,7 +2665,7 @@ bool KateViewBar::hasPermanentWidget(KateViewBarWidget *barWidget) const
 
 void KateViewBar::showBarWidget(KateViewBarWidget *barWidget)
 {
-    Q_ASSERT(barWidget != 0);
+    Q_ASSERT(barWidget != nullptr);
 
     if (barWidget != qobject_cast<KateViewBarWidget *>(m_stack->currentWidget())) {
         hideCurrentBarWidget();
